use cfg_if::cfg_if;
use lazy_static::lazy_static;
use scraper::{ElementRef, Html, Selector};
use std::borrow::Cow;
use std::convert::TryFrom;
use std::error::Error;

mod native;

cfg_if! {
    if #[cfg(feature = "ffi")] {
        mod ffi;
        pub use ffi::*;
        use rmp_serde::{to_vec, to_vec_named};
        use serde::Serialize;
    } else {
        pub use native::*;
   }
}


#[derive(Debug)]
#[cfg_attr(feature = "ffi", derive(Serialize))]
pub struct GResult {
    title: String,
    link: String,
    description: String,
}

//selectors
lazy_static! {
    static ref LINK_SELECT: Selector =
        Selector::parse("div.mw div.col .bkWMgd div.srg div.g div div.rc div.r a cite.iUh30")
            .unwrap();
    static ref TITLE_SELECT: Selector = Selector::parse(".LC20lb").unwrap();
    static ref DESCRIPTION_SELECT: Selector = Selector::parse("div.s").unwrap();
}

// Main parser logic
impl TryFrom<ElementRef<'_>> for GResult {
    type Error = Box<dyn Error + Send + Sync>;
    fn try_from(value: ElementRef) -> Result<Self, Self::Error> {
        let srch_result = value
            .ancestors()
            .nth(4)
            .and_then(ElementRef::wrap)
            .ok_or("Couldnt get search node")?;
        let title = srch_result
            .select(&TITLE_SELECT)
            .next()
            .ok_or("Cant get title")?
            .text()
            .collect::<String>();
        let link = value.text().collect::<String>();
        let description = srch_result
            .select(&DESCRIPTION_SELECT)
            .next()
            .ok_or("Cant get description")?
            .text()
            .collect::<String>();
        Ok(GResult {
            title,
            link,
            description,
        })
    }
}
